import pycurl
import urllib
import xmltodict
import time
import traceback
import threading
import signal
import sys
from StringIO import StringIO

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode
    
class ARDISubscription:
    def __init__(self,core):
        self.core = core
        self.subscription = ""
        self.codes = []
        self.cancelled = False
        self.codechange = False
        self.threaded = True
        self.callback = None
        self.context = None
        self.closed = False

    def AddCode(self,address):
        self.codes.append(address)
        self.codechange = True

    def Connect(self):
        self.ThreadBody()

    def Subscribe(self):
        self._call("subscribe")
        self.codechange = False
        if self.subscription != "":
            return True
        
        return False
    
    def SetCallback(self,call,cont):
        self.callback = call
        self.context = cont

    def Unsubscribe(self):
        self._call("unsubscribe")
        self.subscription = ""
        pass

    def Update(self):
        if self.codechange == True:
            self.Unsubscribe()
            self.Subscribe()
            return
        self._call("update")
        pass

    def catchsignal(self, sig, frame):
        print "ARDI API Caught Interrupt Signal from CURL"
        self.cancelled = True
        signal.signal(signal.SIGINT,signal.SIG_DFL)
        signal.signal(signal.SIGTERM,signal.SIG_DFL)
        raise KeyboardInterrupt("CTRL-C!")
        #sys.exit(0)
    
    def _call(self,function):
        #print "Calling Consolidator " + function
        try:
            #print "  CURL Request"
            fullurl = self.core.server
            try:
                ps = fullurl.index(':')
                if ps > -1:
                    fullurl = fullurl[0:ps]
            except:
                pass

            try:
                ps = fullurl.index('/')
                if ps > -1:
                    fullurl = fullurl[0:ps]
            except:
                pass

            fullurl = "http://" + fullurl
            fullurl += ":" + self.core.contexts[0].consolidator
            fullurl += "/" + function

            #print fullurl

            anydata = False
            post_data = {}
            if function != "subscribe":
                post_data['id'] = self.subscription
                anydata = True

            if function == "subscribe":
                codelist = ""
                for itm in self.codes:
                    #print "Loading Code..."
                    if codelist != "":
                        codelist = codelist + ","
                    codelist += itm
                    anydata = True
                post_data['codes'] = codelist
                anydata = True

            if anydata == True:
                postfields = urlencode(post_data)

            #print fullurl
            #print str(postfields)
            
            try:
                response = StringIO()
                
                c = pycurl.Curl()
                c.setopt(c.URL,fullurl)
                c.setopt(c.WRITEFUNCTION,response.write)
                c.setopt(c.TIMEOUT,30)
                c.setopt(c.CONNECTTIMEOUT,5)
                c.setopt(c.POSTFIELDS,postfields)
                signal.signal(signal.SIGINT,self.catchsignal)
                signal.signal(signal.SIGTERM,self.catchsignal)
                c.perform()
                c.close()
                signal.signal(signal.SIGINT,signal.SIG_DFL)
                signal.signal(signal.SIGTERM,signal.SIG_DFL)
                
                returned = {}
                
                #print response.getvalue()

                result = xmltodict.parse(response.getvalue())
                
                self.subscription = result['update']['id']
                try:
                    returned[result['update']['points']['point']['@code']] = result['update']['points']['point']['@value']
                except:
                    try:
                        for p in result['update']['points']['point']:
                            #print str(p)
                            returned[p['@code']] = p['@value']
                    except:
                        pass

                #print "Calling Back - " + str(returned)
                if self.callback is not None:
                    self.callback(returned,self.context)
            except pycurl.error as e:
                if e[0] == 23:
                    self.closed = True
                    #print "Marking Closed!"
                    return False
                if e[0] != 28:
                    traceback.print_exc()
                    return False
                else:
                    #print "Response Timeout"
                    return True
            except:
                print "Failed To Send!"
                traceback.print_exc()
                return False
            return True
        
        except:
            traceback.print_exc()
            return False

    def ThreadBody(self):
        while self.cancelled == False:
            while self.subscription == "":
                if self.Subscribe() == False:
                    time.sleep(5)
                    if self.cancelled == True:
                        break;
                else:
                    break
            if self.cancelled == True:
                break;

            if self.Update():
                #Call the callback function with our new data...
                time.sleep(1)
                pass
            else:
                #No new data arrived - immediately try again.
                time.sleep(0.5)
                pass